/**
 * Borrow from https://github.com/excalidraw/excalidraw/blob/master/packages/excalidraw/change.ts#L399
 */
import { Delta } from './Delta';
export class AppStateChange {
    constructor(delta) {
        this.delta = delta;
    }
    static empty() {
        return new AppStateChange(Delta.create({}, {}));
    }
    static calculate(prevAppState, nextAppState) {
        const delta = Delta.calculate(prevAppState, nextAppState, undefined);
        return new AppStateChange(delta);
    }
    // private static postProcess<T extends ObservedAppState>(
    //   deleted: Partial<T>,
    //   inserted: Partial<T>,
    // ): [Partial<T>, Partial<T>] {
    //   try {
    //     Delta.diffObjects(
    //       deleted,
    //       inserted,
    //       "selectedElementIds",
    //       // ts language server has a bit trouble resolving this, so we are giving it a little push
    //       (_) => true as ValueOf<T["selectedElementIds"]>,
    //     );
    //     Delta.diffObjects(
    //       deleted,
    //       inserted,
    //       "selectedGroupIds",
    //       (prevValue) => (prevValue ?? false) as ValueOf<T["selectedGroupIds"]>,
    //     );
    //   } catch (e) {
    //     // if postprocessing fails it does not make sense to bubble up, but let's make sure we know about it
    //     console.error(`Couldn't postprocess appstate change deltas.`);
    //   } finally {
    //     return [deleted, inserted];
    //   }
    // }
    inverse() {
        const inversedDelta = Delta.create(this.delta.inserted, this.delta.deleted);
        return new AppStateChange(inversedDelta);
    }
    applyTo(appState, nextElements) {
        // TODO: selected elements
        const directlyApplicablePartial = this.delta.inserted;
        const nextAppState = Object.assign(Object.assign({}, appState), directlyApplicablePartial);
        const constainsVisibleChanges = this.filterInvisibleChanges(appState, nextAppState, nextElements);
        return [nextAppState, constainsVisibleChanges];
    }
    /**
     * Mutates `nextAppState` be filtering out state related to deleted elements.
     *
     * @returns `true` if a visible change is found, `false` otherwise.
     */
    filterInvisibleChanges(prevAppState, nextAppState, nextElements) {
        return true;
    }
    isEmpty() {
        return Delta.isEmpty(this.delta);
    }
}
//# sourceMappingURL=AppStateChange.js.map