"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _API_canvasEntity, _API_idEntityMap, _API_history, _API_store;
Object.defineProperty(exports, "__esModule", { value: true });
exports.API = exports.pendingCanvases = void 0;
const ecs_1 = require("@infinite-canvas-tutorial/ecs");
const event_1 = require("./event");
const components_1 = require("./components");
const history_1 = require("./history");
const utils_1 = require("./utils");
/**
 * Since the canvas is created in the system, we need to store them here for later use.
 */
exports.pendingCanvases = [];
/**
 * Expose the API to the outside world.
 *
 * @see https://docs.excalidraw.com/docs/@excalidraw/excalidraw/api/props/excalidraw-api
 */
class API {
    constructor(element, commands) {
        this.element = element;
        this.commands = commands;
        _API_canvasEntity.set(this, void 0);
        _API_idEntityMap.set(this, new Map());
        _API_history.set(this, new history_1.History());
        _API_store.set(this, new history_1.Store(this));
        __classPrivateFieldGet(this, _API_store, "f").onStoreIncrementEmitter.on(history_1.StoreIncrementEvent, (event) => {
            var _a;
            __classPrivateFieldGet(this, _API_history, "f").record(event.elementsChange, event.appStateChange);
            (_a = this.onchange) === null || _a === void 0 ? void 0 : _a.call(this, __classPrivateFieldGet(this, _API_store, "f").snapshot);
        });
    }
    getElement() {
        return this.element;
    }
    getEntityCommands() {
        return __classPrivateFieldGet(this, _API_idEntityMap, "f");
    }
    getEntity(node) {
        return __classPrivateFieldGet(this, _API_idEntityMap, "f").get(node.id).id();
    }
    getNodeById(id) {
        return this.getNodes().find((node) => node.id === id);
    }
    /**
     * Create a new canvas.
     */
    createCanvas(canvasProps) {
        const canvas = this.commands.spawn(new ecs_1.Canvas(canvasProps)).id();
        canvas.add(components_1.Container, { element: this.element });
        __classPrivateFieldSet(this, _API_canvasEntity, canvas.hold(), "f");
    }
    /**
     * Create a new camera.
     */
    createCamera(cameraProps) {
        const { zoom } = cameraProps;
        this.commands.spawn(new ecs_1.Camera({
            canvas: __classPrivateFieldGet(this, _API_canvasEntity, "f"),
        }), new ecs_1.Transform({
            scale: {
                x: 1 / zoom,
                y: 1 / zoom,
            },
        }));
    }
    /**
     * Create a new landmark.
     */
    createLandmark() { }
    /**
     * Go to a landmark.
     */
    gotoLandmark() { }
    /**
     * ZoomLevel system will handle the zoom level.
     * @see https://infinitecanvas.cc/guide/lesson-004
     */
    zoomTo(zoom) {
        this.element.dispatchEvent(new CustomEvent(event_1.Event.ZOOM_TO, {
            detail: {
                zoom,
            },
        }));
    }
    /**
     * Resize the canvas.
     * @see https://infinitecanvas.cc/guide/lesson-001
     */
    resizeCanvas(width, height) {
        Object.assign(__classPrivateFieldGet(this, _API_canvasEntity, "f").write(ecs_1.Canvas), {
            width,
            height,
        });
        this.element.dispatchEvent(new CustomEvent(event_1.Event.RESIZED, {
            detail: { width, height },
        }));
    }
    /**
     * Set the checkboard style.
     * @see https://infinitecanvas.cc/guide/lesson-005
     */
    setCheckboardStyle(checkboardStyle) {
        Object.assign(__classPrivateFieldGet(this, _API_canvasEntity, "f").write(ecs_1.Grid), {
            checkboardStyle,
        });
        const prevAppState = this.getAppState();
        this.setAppState(Object.assign(Object.assign({}, prevAppState), { checkboardStyle }));
        this.element.dispatchEvent(new CustomEvent(event_1.Event.CHECKBOARD_STYLE_CHANGED, {
            detail: { checkboardStyle },
        }));
    }
    setPen(pen) {
        Object.assign(__classPrivateFieldGet(this, _API_canvasEntity, "f").write(ecs_1.Canvas), {
            pen,
        });
        const prevAppState = this.getAppState();
        this.setAppState(Object.assign(Object.assign({}, prevAppState), { penbarSelected: [pen] }));
        this.element.dispatchEvent(new CustomEvent(event_1.Event.PEN_CHANGED, {
            detail: { selected: [pen] },
        }));
    }
    setTaskbars(selected) {
        const prevAppState = this.getAppState();
        this.setAppState(Object.assign(Object.assign({}, prevAppState), { taskbarSelected: selected }));
        this.element.dispatchEvent(new CustomEvent(event_1.Event.TASK_CHANGED, {
            detail: { selected },
        }));
    }
    setPropertiesOpened(propertiesOpened) {
        const prevAppState = this.getAppState();
        this.setAppState(Object.assign(Object.assign({}, prevAppState), { propertiesOpened }));
    }
    /**
     * @see https://docs.excalidraw.com/docs/@excalidraw/excalidraw/api/props/excalidraw-api#setcursor
     */
    setCursor(cursor) {
        if (!__classPrivateFieldGet(this, _API_canvasEntity, "f").has(ecs_1.Cursor)) {
            __classPrivateFieldGet(this, _API_canvasEntity, "f").add(ecs_1.Cursor);
        }
        Object.assign(__classPrivateFieldGet(this, _API_canvasEntity, "f").write(ecs_1.Cursor), {
            value: cursor,
        });
    }
    /**
     * Delete Canvas component
     */
    destroy() {
        __classPrivateFieldGet(this, _API_canvasEntity, "f").delete();
        this.element.dispatchEvent(new CustomEvent(event_1.Event.DESTROY));
    }
    /**
     * Select nodes.
     */
    selectNodes(selected, preserveSelection = false) {
        const prevAppState = this.getAppState();
        if (!preserveSelection) {
            this.getAppState().layersSelected.forEach((id) => {
                var _a;
                const entity = (_a = __classPrivateFieldGet(this, _API_idEntityMap, "f").get(id)) === null || _a === void 0 ? void 0 : _a.id();
                if (entity) {
                    entity.remove(ecs_1.Selected);
                }
            });
        }
        this.setAppState(Object.assign(Object.assign({}, prevAppState), { layersSelected: preserveSelection
                ? [...prevAppState.layersSelected, ...selected]
                : selected }));
        // Select nodes in the canvas.
        this.getAppState().layersSelected.forEach((id) => {
            var _a;
            const entity = (_a = __classPrivateFieldGet(this, _API_idEntityMap, "f").get(id)) === null || _a === void 0 ? void 0 : _a.id();
            if (entity && !entity.has(ecs_1.Selected)) {
                entity.add(ecs_1.Selected);
            }
        });
        this.element.dispatchEvent(new CustomEvent(event_1.Event.SELECTED_NODES_CHANGED, {
            detail: { selected, preserveSelection },
        }));
    }
    /**
     * If diff is provided, no need to calculate diffs.
     */
    updateNode(node, diff) {
        var _a;
        const entity = (_a = __classPrivateFieldGet(this, _API_idEntityMap, "f").get(node.id)) === null || _a === void 0 ? void 0 : _a.id();
        const nodes = this.getNodes();
        if (!entity) {
            const { cameras } = __classPrivateFieldGet(this, _API_canvasEntity, "f").read(ecs_1.Canvas);
            if (cameras.length === 0) {
                throw new Error('No camera found');
            }
            // TODO: Support multiple cameras.
            const camera = cameras[0];
            const cameraEntityCommands = this.commands.entity(camera);
            // TODO: Calculate diffs and only update the changed nodes.
            const { entities, idEntityMap } = (0, ecs_1.serializedNodesToEntities)([node], this.commands);
            __classPrivateFieldGet(this, _API_idEntityMap, "f").set(node.id, idEntityMap.get(node.id));
            this.commands.execute();
            entities.forEach((entity) => {
                // Append roots to the camera.
                if (!entity.has(ecs_1.Children)) {
                    cameraEntityCommands.appendChild(this.commands.entity(entity));
                }
            });
            this.commands.execute();
            this.setNodes([...nodes, node]);
            this.element.dispatchEvent(new CustomEvent(event_1.Event.NODE_UPDATED, {
                detail: {
                    node,
                },
            }));
        }
        else {
            const updated = (0, history_1.mutateElement)(entity, node, diff);
            const index = nodes.findIndex((n) => n.id === updated.id);
            if (index !== -1) {
                nodes[index] = updated;
                this.setNodes(nodes);
            }
            this.element.dispatchEvent(new CustomEvent(event_1.Event.NODE_UPDATED, {
                detail: {
                    node: updated,
                },
            }));
        }
    }
    /**
     * Update the scene with new nodes.
     * It will calculate diffs and only update the changed nodes.
     *
     * @see https://docs.excalidraw.com/docs/@excalidraw/excalidraw/api/props/excalidraw-api#updatescene
     */
    updateNodes(nodes) {
        const existentNodes = nodes.filter((node) => __classPrivateFieldGet(this, _API_idEntityMap, "f").has(node.id));
        const nonExistentNodes = nodes.filter((node) => !__classPrivateFieldGet(this, _API_idEntityMap, "f").has(node.id));
        if (nonExistentNodes.length > 0) {
            const { cameras } = __classPrivateFieldGet(this, _API_canvasEntity, "f").read(ecs_1.Canvas);
            if (cameras.length === 0) {
                throw new Error('No camera found');
            }
            // TODO: Support multiple cameras.
            const camera = cameras[0];
            const cameraEntityCommands = this.commands.entity(camera);
            // TODO: Calculate diffs and only update the changed nodes.
            const { entities, idEntityMap } = (0, ecs_1.serializedNodesToEntities)(nonExistentNodes, this.commands);
            nonExistentNodes.forEach((node) => {
                __classPrivateFieldGet(this, _API_idEntityMap, "f").set(node.id, idEntityMap.get(node.id));
            });
            this.commands.execute();
            entities.forEach((entity) => {
                // Append roots to the camera.
                if (!entity.has(ecs_1.Children)) {
                    cameraEntityCommands.appendChild(this.commands.entity(entity));
                }
            });
            this.commands.execute();
            this.setNodes([...this.getNodes(), ...nonExistentNodes]);
        }
        if (existentNodes.length > 0) {
            existentNodes.forEach((node) => {
                this.updateNode(node);
            });
        }
        this.element.dispatchEvent(new CustomEvent(event_1.Event.NODES_UPDATED, {
            detail: {
                nodes,
            },
        }));
    }
    deleteNodesById(ids) {
        const nodes = this.getNodes();
        const index = nodes.findIndex((n) => ids.includes(n.id));
        if (index !== -1) {
            nodes.splice(index, 1);
        }
        ids.forEach((id) => {
            const entity = __classPrivateFieldGet(this, _API_idEntityMap, "f").get(id);
            if (entity) {
                // entity.id().delete();
                entity.id().add(ecs_1.ToBeDeleted);
            }
            __classPrivateFieldGet(this, _API_idEntityMap, "f").delete(id);
        });
        this.setNodes(nodes);
        this.element.dispatchEvent(new CustomEvent(event_1.Event.NODE_DELETED, {
            detail: {
                nodes,
            },
        }));
    }
    /**
     * Record the current state of the canvas.
     * @param captureUpdateAction Record the changes immediately or never.
     */
    record(captureUpdateAction = history_1.CaptureUpdateAction.IMMEDIATELY) {
        if (captureUpdateAction === history_1.CaptureUpdateAction.NEVER ||
            __classPrivateFieldGet(this, _API_store, "f").snapshot.isEmpty()) {
            __classPrivateFieldGet(this, _API_store, "f").shouldUpdateSnapshot();
        }
        else {
            __classPrivateFieldGet(this, _API_store, "f").shouldCaptureIncrement();
        }
        __classPrivateFieldGet(this, _API_store, "f").commit((0, utils_1.arrayToMap)(this.getNodes()), this.getAppState());
    }
    undo() {
        const result = __classPrivateFieldGet(this, _API_history, "f").undo((0, utils_1.arrayToMap)(this.getNodes()), this.getAppState(), __classPrivateFieldGet(this, _API_store, "f").snapshot);
        if (result) {
            const [elements, appState] = result;
            this.setNodes((0, utils_1.mapToArray)(elements));
            this.setAppState(appState);
        }
    }
    redo() {
        const result = __classPrivateFieldGet(this, _API_history, "f").redo((0, utils_1.arrayToMap)(this.getNodes()), this.getAppState(), __classPrivateFieldGet(this, _API_store, "f").snapshot);
        if (result) {
            const [elements, appState] = result;
            this.setNodes((0, utils_1.mapToArray)(elements));
            this.setAppState(appState);
        }
    }
    isUndoStackEmpty() {
        return __classPrivateFieldGet(this, _API_history, "f").isUndoStackEmpty;
    }
    isRedoStackEmpty() {
        return __classPrivateFieldGet(this, _API_history, "f").isRedoStackEmpty;
    }
    clearHistory() {
        __classPrivateFieldGet(this, _API_history, "f").clear();
    }
}
exports.API = API;
_API_canvasEntity = new WeakMap(), _API_idEntityMap = new WeakMap(), _API_history = new WeakMap(), _API_store = new WeakMap();
//# sourceMappingURL=API.js.map