"use strict";
/**
 * Borrow from https://github.com/excalidraw/excalidraw/blob/master/packages/excalidraw/store.ts
 */
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _Store_snapshot, _Store_scheduledActions;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Store = exports.CaptureUpdateAction = exports.StoreIncrementEvent = void 0;
const eventemitter3_1 = require("eventemitter3");
const AppStateChange_1 = require("./AppStateChange");
const ElementsChange_1 = require("./ElementsChange");
const Snapshot_1 = require("./Snapshot");
exports.StoreIncrementEvent = 'storeIncrement';
exports.CaptureUpdateAction = {
    /**
     * Immediately undoable.
     *
     * Use for updates which should be captured.
     * Should be used for most of the local updates.
     *
     * These updates will _immediately_ make it to the local undo / redo stacks.
     */
    IMMEDIATELY: 'IMMEDIATELY',
    /**
     * Never undoable.
     *
     * Use for updates which should never be recorded, such as remote updates
     * or scene initialization.
     *
     * These updates will _never_ make it to the local undo / redo stacks.
     */
    NEVER: 'NEVER',
    /**
     * Eventually undoable.
     *
     * Use for updates which should not be captured immediately - likely
     * exceptions which are part of some async multi-step process. Otherwise, all
     * such updates would end up being captured with the next
     * `CaptureUpdateAction.IMMEDIATELY` - triggered either by the next `updateScene`
     * or internally by the editor.
     *
     * These updates will _eventually_ make it to the local undo / redo stacks.
     */
    EVENTUALLY: 'EVENTUALLY',
};
class Store {
    constructor(api) {
        this.api = api;
        _Store_snapshot.set(this, Snapshot_1.Snapshot.empty());
        _Store_scheduledActions.set(this, new Set());
        this.onStoreIncrementEmitter = new eventemitter3_1.EventEmitter();
    }
    get snapshot() {
        return __classPrivateFieldGet(this, _Store_snapshot, "f");
    }
    set snapshot(snapshot) {
        __classPrivateFieldSet(this, _Store_snapshot, snapshot, "f");
    }
    scheduleAction(action) {
        __classPrivateFieldGet(this, _Store_scheduledActions, "f").add(action);
    }
    shouldCaptureIncrement() {
        this.scheduleAction(exports.CaptureUpdateAction.IMMEDIATELY);
    }
    shouldUpdateSnapshot() {
        this.scheduleAction(exports.CaptureUpdateAction.NEVER);
    }
    commit(elements, appState) {
        try {
            // Capture has precedence since it also performs update
            if (__classPrivateFieldGet(this, _Store_scheduledActions, "f").has(exports.CaptureUpdateAction.IMMEDIATELY)) {
                this.captureIncrement(elements, appState);
            }
            else if (__classPrivateFieldGet(this, _Store_scheduledActions, "f").has(exports.CaptureUpdateAction.NEVER)) {
                this.updateSnapshot(elements, appState);
            }
        }
        finally {
            // Defensively reset all scheduled actions, potentially cleans up other runtime garbage
            __classPrivateFieldSet(this, _Store_scheduledActions, new Set(), "f");
        }
    }
    captureIncrement(elements, appState) {
        const prevSnapshot = this.snapshot;
        const nextSnapshot = this.snapshot.maybeClone(elements, appState);
        // Optimisation, don't continue if nothing has changed
        if (prevSnapshot !== nextSnapshot) {
            // Calculate and record the changes based on the previous and next snapshot
            const elementsChange = nextSnapshot.meta.didElementsChange
                ? ElementsChange_1.ElementsChange.calculate(prevSnapshot.elements, nextSnapshot.elements, this.api)
                : ElementsChange_1.ElementsChange.empty();
            const appStateChange = nextSnapshot.meta.didAppStateChange
                ? AppStateChange_1.AppStateChange.calculate(prevSnapshot.appState, nextSnapshot.appState)
                : AppStateChange_1.AppStateChange.empty();
            if (!elementsChange.isEmpty() || !appStateChange.isEmpty()) {
                // Notify listeners with the increment
                this.onStoreIncrementEmitter.emit(exports.StoreIncrementEvent, {
                    elementsChange,
                    appStateChange,
                });
            }
            // Update snapshot
            this.snapshot = nextSnapshot;
        }
    }
    updateSnapshot(elements, appState) {
        const nextSnapshot = this.snapshot.maybeClone(elements, appState);
        if (this.snapshot !== nextSnapshot) {
            // Update snapshot
            this.snapshot = nextSnapshot;
        }
    }
}
exports.Store = Store;
_Store_snapshot = new WeakMap(), _Store_scheduledActions = new WeakMap();
//# sourceMappingURL=Store.js.map